package com.zefie.zkiosk

import android.content.Context
import android.net.Uri
import android.os.Handler
import android.os.Looper
import android.widget.Toast
import org.videolan.libvlc.LibVLC
import org.videolan.libvlc.Media
import org.videolan.libvlc.MediaPlayer
import org.videolan.libvlc.util.VLCVideoLayout

class VLCPlayer(private val context: Context, private val videoLayout: VLCVideoLayout, private val streamId: Int) {
    private val libVLC: LibVLC = LibVLC(context, arrayListOf("--rtsp-tcp",
        "--quiet",
        "--no-drop-late-frames",
        "--no-skip-frames",
        "--verbose=0"))
    private val mediaPlayer: MediaPlayer = MediaPlayer(libVLC)
    private var streamUrl: String? = null
    private val handler = Handler(Looper.getMainLooper())
    private var lastPlaybackTime: Long = 0
    private var lastProgressCheck: Long = 0
    private var watchdogRunnable: Runnable? = null

    init {
        mediaPlayer.attachViews(videoLayout, null, false, true)
        // Listen for errors
        mediaPlayer.setEventListener { event ->
            if (event.type == MediaPlayer.Event.EncounteredError ||
                event.type == MediaPlayer.Event.EndReached) {
                restartStream()
            }
        }
    }

    fun playRTSP(url: String) {
        streamUrl = url
        val media = Media(libVLC, Uri.parse(url))
        media.setHWDecoderEnabled(true, false)
        media.addOption(":network-caching=500") // Adjust caching to handle jitter
        mediaPlayer.media = media
        media.release()
        mediaPlayer.play()
        mediaPlayer.volume = 0
        startWatchdog()
    }

    private fun startWatchdog() {
        lastPlaybackTime = mediaPlayer.time
        lastProgressCheck = System.currentTimeMillis()

        watchdogRunnable = object : Runnable {
            override fun run() {
                val currentTime = mediaPlayer.time
                val currentPosition = mediaPlayer.position
                val stats = mediaPlayer.media?.stats
                val decodedVideo = stats?.decodedVideo ?: 0
                val now = System.currentTimeMillis()

                val noTimeProgress = currentTime == lastPlaybackTime || currentTime == 0L
                val noPositionProgress = currentPosition == 0f
                val noFramesDecoded = decodedVideo == 0
                val stalled = noTimeProgress && noPositionProgress && noFramesDecoded

                if (stalled) {
                    val freezeDuration = now - lastProgressCheck
                    if (freezeDuration > 8000) {
                        handler.post {
                            Toast.makeText(context, "Watchdog triggered: Stream ${streamId} stalled, restarting...", Toast.LENGTH_SHORT).show()
                        }
                        restartStream()
                        return
                    }
                } else {
                    lastProgressCheck = now
                    lastPlaybackTime = currentTime
                }

                handler.postDelayed(this, 3000)
            }
        }

        handler.post(watchdogRunnable!!)
    }

    private fun restartStream() {
        if (streamUrl != null) {
            handler.postDelayed({
                mediaPlayer.stop()
                playRTSP(streamUrl!!) // Restart the stream
            }, 2000) // Wait 2 seconds before restarting
        }
    }

    var volume: Int
        get() = mediaPlayer.volume
        set(value) {
            mediaPlayer.volume = value
        }

    fun stop() {
        watchdogRunnable?.let { handler.removeCallbacks(it) }
        mediaPlayer.stop()
    }

    fun release() {
        watchdogRunnable?.let { handler.removeCallbacks(it) }
        mediaPlayer.release()
        libVLC.release()
    }
}
